package edu.georgetown.gucs.dictionary;

/**
 * Creates a {@linkplain Dictionary} containing tokens with IDFs within a specified range
 *
 * @author Clay Shields
 */
public class TrimDictionary {

    private final Dictionary dictionary;
    private double min_idf = 0.0;
    private double max_idf = 1.0;

    /**
     * Constructor that specifies the IDF range and dictionary to trim
     *
     * @param min_idf
     *          the <code>double</code> minimum normalized inverse document frequency (IDF) to use for this trimmed
     *          dictionary
     * @param max_idf
     *          the <code>double</code> maximum normalized inverse document frequency (IDF) to use for this trimmed
     *          dictionary
     * @param dictionary_file
     *          the string filename containing the dictionary to trim
     **/
    public TrimDictionary(double min_idf, double max_idf, String dictionary_file) {
        this.dictionary = new Dictionary(dictionary_file);
        //System.err.println("TrimDictionary read dictionary of size: " + dictionary.size());
        this.min_idf = min_idf;
        this.max_idf = max_idf;
    }

    /**
     * Trims the dictionary and saves it to the given output file
     *
     * @param output_file
     *          the string filename to write the trimmed dictionary to
     */
    public void trim(String output_file) {
        this.dictionary.trimByIDF(this.min_idf, this.max_idf);
        this.dictionary.saveDictionaryXML(output_file);
    }

}
