package edu.georgetown.gucs.matcher;

import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.fingerprinter.Fingerprint;
import edu.georgetown.gucs.utility.FileLister;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.Pair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.*;

/**
 * @author Evan B.
 * @since 12/17/14.
 * Read a directory of serialized fingerprints;
 * Convert serialized fingerprints to a fingerprint list
 * Run bulkMatcher on the fingerprint list
 * Graph {Figure out a method for graphing}
 */
public class ExperimentN2 {


    private final SortedMap<Integer, List<String>> scores;
    private final int maxThread;
    private final String matcher;

    private String trialsDirectory;
    private List<Fingerprint> recoveredFingerprints = new ArrayList<>();
    private List<Pair<String, Integer>> resultSet;
    private boolean update;


    private static final Logger logger = LoggerFactory.getLogger(ExperimentN2.class);

    /**
     * Comparator that orders scores highest to lowest
     */
    private static class HightoLowScore implements Comparator<Integer>, Serializable {
        private static final long serialVersionUID = 5679507537454070748L;

        public HightoLowScore() {
        }

        @Override
        public int compare(Integer one, Integer two) {
            return two.compareTo(one);
        }
    }

    public ExperimentN2(String trialsDirectory,
                        String matcherName,
                        boolean update) {

        this.scores = Collections.synchronizedSortedMap(new TreeMap<>(new HightoLowScore()));
        this.matcher = matcherName;
        this.maxThread = Runtime.getRuntime().availableProcessors();
        this.update = update;

        this.trialsDirectory = trialsDirectory;


        System.out.println("ExperimentN2");
        readDirectory();
        n2Comparison();

    }

    private void readDirectory() {

        FileLister expTrialDir = new FileLister(trialsDirectory);

        for (File files : expTrialDir.getFileListerArray()) {

            try (
                    InputStream file = new FileInputStream(files.getAbsolutePath());
                    InputStream buffer = new BufferedInputStream(file);
                    ObjectInput input = new ObjectInputStream(buffer)
            ) {

                if (files.getName().contains("dict")) {

                    System.out.println("Files Dict: " + files.getAbsolutePath());
                    Dictionary dict = (Dictionary) input.readObject();

                } else if (!files.getName().contains("dict")) {
                    //deserialize the List

                    System.out.println("Files: " + files.getAbsolutePath());

                    @SuppressWarnings("unchecked")
                    List<Fingerprint> temp = (List<Fingerprint>) input.readObject();

                    recoveredFingerprints.addAll(temp);

                }

            } catch (ClassNotFoundException ex) {
                System.out.println("Cannot perform input. Class not found." + ex);
            } catch (IOException ex) {
                System.out.println("Cannot perform input." + ex);
            }

        }
    }

    private void n2Comparison() {

        BulkMatcher bulkMatch = new BulkMatcher(matcher, scores, maxThread);
        Global.debug("fingSize", recoveredFingerprints.size());

        if(update) {
            bulkMatch.updateStdOut();
        }

        //format [string::file1|file2|integer::score]
        resultSet = bulkMatch.compareAllFingerprints(recoveredFingerprints);

    }

    public void printScores() {

        for (Pair<String, Integer> results : this.resultSet) {

            System.out.println(results.getA() + "|" + results.getB());

        }

    }


    public void saveScores(String fileName) {

        System.out.println("Writing Scores to " + fileName);
        try (PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(fileName, true)))) {

            for (Pair<String, Integer> results : resultSet) {

                out.println(results.getA() + "|" + results.getB());

            }
        } catch (IOException e) {
            //exception handling left as an exercise for the reader
        }
    }

    public void persist(){

    }
}