package edu.georgetown.gucs.matcher;

import edu.georgetown.gucs.utility.Global;
import org.apache.xerces.impl.io.UTF8Reader;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.Random;
import java.util.logging.Logger;

/**
 * @author Evan Bloomberg
 *         created on 5/5/14.
 */
public class SdHashFingerprintMatcher extends FingerprintMatcher {


    private final String threadName = Thread.currentThread().getName().replaceAll("\\s", "");
    private final Random generator = new Random(System.currentTimeMillis());
    private String sdHashCmpScore;

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(SdHashFingerprintMatcher.class);

    /**
     * Returns a random number
     *
     * @return a random double based on @value generators value
     */
    private String randomGenerator() {
        return String.valueOf(generator.nextDouble() * 0.5);
    }


    /**
     * The constructor that is called to inherit the Matcher parent class
     */
    public SdHashFingerprintMatcher() {
        super();
    }

    /**
     * Takes two Lists that are of type Pair where the fingerprint is stored in byte form
     *
     * @param fingerprint1 the list that contains the first fingerprint to compare
     * @param fingerprint2 list that contains the second fingerprint to compare
     * @return the score of two files compared by SdHash
     */
    @Override
    public int getScore(String fingerprint1, String fingerprint2) {
        // There is only one fingerprint per list {located at the 0 element}


        String fileName = threadName + randomGenerator();

        Integer score = 0;
        try {
            //create
            File tempFile = File.createTempFile(fileName, ".sdbf");


            try (BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(tempFile),Global.fileEncoding))) {
                bufferedWriter.write(fingerprint1);
                bufferedWriter.write(fingerprint2);

            }

            String minScore = String.valueOf(getMinimumScore());


            try (InputStream inStream = new ProcessBuilder(Global.sdHashFilePath, "-c", tempFile.getCanonicalPath(), "-t", minScore)
                    .redirectErrorStream(true).start().getInputStream();
                 BufferedReader buffer = new BufferedReader(new InputStreamReader(inStream, Global.fileEncoding))) {

                String line;
                StringBuilder sdHashComparisonScore = new StringBuilder();
                while ((line = buffer.readLine()) != null) {
                    sdHashComparisonScore.append(line).append("\n");
                }

                boolean deleteFile = tempFile.delete();

                sdHashCmpScore = sdHashComparisonScore.toString();

                if(sdHashCmpScore.isEmpty()) {
                    return 0;
                }

                String tempScore = sdHashComparisonScore.substring(sdHashComparisonScore.lastIndexOf("|") + 1,
                        sdHashComparisonScore.length()).trim();

                if (tempScore.equalsIgnoreCase("100")) {

                    score = 100;

                } else {

                    try {
                        score = Integer.parseInt(tempScore.substring(1, tempScore.length()));
                    } catch (NumberFormatException e) {
//                        System.out.println("Error: " + tempScore.substring(1, tempScore.length()));
                        Global.debug("ERROR", tempScore);
                    }
                }
            }

        } catch (IOException e) {
            e.printStackTrace();
        }


        return score;
    }

    public void printScore() {

        System.out.println("********Fingerprint Score*********");
        System.out.println(sdHashCmpScore);

    }


    @Override
    public String toString() {
        return "SdHashFingerprintMatcher";
    }


}
