package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.stream.Collectors;

import edu.georgetown.gucs.utility.Token;

/**
 * Eliminates tokens that are longer than a given length
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 *
 */
public class MaximumLengthTokenizer extends Tokenizer {

    private static final long serialVersionUID = -1057327977380694005L;
    private final int maxTokenLength;

    /**
     * Constructor that sets the maximum token length to be considered
     *
     * @param length
     *          the string value of the maximum token length to be included
     */
    public MaximumLengthTokenizer(String length) {
        this.maxTokenLength = Integer.parseInt(length);
    }

    /**
     * Eliminates tokens that are longer than the length specified in the constructor
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) {
        List<Token> tokenVector = Collections.synchronizedList(new Vector<>());
        synchronized (tokenVector) {
            tokenVector.addAll(tokens.stream().filter(token -> token.name().length() <= this.maxTokenLength).collect(Collectors.toList()));
            return tokenVector;
        }
    }
}
