package edu.georgetown.gucs.tokenizers;

import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.Token;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.zip.GZIPInputStream;

public class ParseTokenizers extends FileTokenizer {

    private StringBuilder token_ = new StringBuilder("");
    private String fileName_;
    private int byte_start_ = 0;
    private int byte_end_ = 0;


    List<Token> tokenVector = Collections.synchronizedList(new Vector<>());

    private boolean isGz;



    public ParseTokenizers(String fileName) {
        this.fileName_ = fileName;
        isGz = fileName.endsWith(".gz");

    }


    public void splitLines() {

        mode_ = "lines";

    }

    public List<Token> readFile() throws IOException {

        try (FileInputStream fis = new FileInputStream(fileName_)) {

            GZIPInputStream gis = null;
            InputStreamReader isr;

            if (isGz) {
                gis = new GZIPInputStream(fis);
                isr = new InputStreamReader(gis, Global.fileEncoding);
            } else {
                isr = new InputStreamReader(fis, Global.fileEncoding);

            }


            try (BufferedReader buffer_reader = new BufferedReader(isr)) {

                synchronized (this.tokenVectorMap) {

                    this.tokenVectorMap.clear();

                    int character_;
                    while ((character_ = buffer_reader.read()) != -1) {

                      createTokens(character_);

                    }

                    this.tokenVectorMap.put("null", tokenVector);
                }
            }
            if (gis != null) {
                gis.close();
            }
            isr.close();

            return tokenVector;
        }
    }

    //TODO change filename to more generic name
    public List<Token> readString(){

       for(Character character : fileName_.toCharArray()) {

           createTokens(character);

       }

        this.tokenVectorMap.put("null", tokenVector);
        return tokenVector;
    }


    public void createTokens(int character_){

        if (isLineFeed(character_)) {

            if (!token_.toString().trim().isEmpty()) {

                tokenVector.add(new Token(token_.toString().trim(), byte_start_, byte_end_));
                byte_start_ = byte_end_;
            }

            byte_end_ += Character.charCount(character_);
            token_.delete(0, token_.length());

        } else {


            token_.append((char) character_);
            byte_end_ += Character.charCount(character_);

        }

    }

    boolean isLineFeed(int tokenChar) {
        return ('\n' == tokenChar || '\r' == tokenChar || '\f' == tokenChar) || (Character.isWhitespace(tokenChar));
    }
}
