package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import edu.georgetown.gucs.utility.Token;

/**
 * Eliminates tokens that are numbers
 *
 * @author Clay Shields
 * @author Lindsay Neubauer
 *
 */
public class RemoveNumericTokensTokenizer extends Tokenizer {

    private static final long serialVersionUID = 4100228420352022457L;
    private static final Pattern isDigitPattern = Pattern.compile("^\\d+$");

    /**
     * Eliminates tokens that are numbers
     */
    @Override
    public List<Token> tokenize(List<Token> tokens) {
        List<Token> tokenVector = Collections.synchronizedList(new Vector<>());
        synchronized (tokenVector) {
            tokenVector.addAll(tokens.stream().filter(token -> !isDigitPattern.matcher(token.name()).matches()).collect(Collectors.toList()));
            return tokenVector;
        }
    }
}
